import re

from django.core.exceptions import ValidationError
from django.utils.safestring import mark_safe
from wagtail.core.blocks import RichTextBlock

FIND_FOOTNOTE_TAG = re.compile(r'<footnote id="(.*?)">.*?</footnote>')


class RichTextBlockWithFootnotes(RichTextBlock):
    """
    Rich Text block that renders footnotes in the format
    '<footnote id="long-id">short-id</footnote>' as anchor elements. It also
    adds the Footnote object to the 'request' object for later use. It uses
    'request' because variables added to 'context' do not persist into the
    final template context.
    """

    def render_basic(self, value, context=None):
        html = super().render_basic(value, context)

        def replace_tag(match):
            try:
                index = self.process_footnote(match.group(1), context["page"])
            except (KeyError, ValidationError):
                return ""
            else:
                return f'<a href="#footnote-{index}" id="footnote-source-{index}"><sup>[{index}]</sup></a>'
        return mark_safe(FIND_FOOTNOTE_TAG.sub(replace_tag, html))

    def process_footnote(self, footnote_id, page):
        footnotes = self.get_footnotes(page)
        footnote = page.footnotes.get(uuid=footnote_id)
        if footnote not in footnotes:
            footnotes.append(footnote)
        return footnotes.index(footnote) + 1

    def get_footnotes(self, page):
        if not hasattr(page, "footnotes_list"):
            page.footnotes_list = []
        return page.footnotes_list



from django.urls import path
from django.views.generic import TemplateView

urlpatterns = [
    path(
        "footnotes_modal/",
        TemplateView.as_view(
            template_name="wagtail_footnotes/admin/footnotes_modal.html"
        ),
        name="footnotes-modal",
    )
]



import wagtail.admin.rich_text.editors.draftail.features as draftail_features
from django.templatetags.static import static
from django.utils.html import format_html_join
from draftjs_exporter.dom import DOM
from wagtail.admin.rich_text.converters.html_to_contentstate import (
    InlineEntityElementHandler,
)
from wagtail.core import hooks


@hooks.register("insert_editor_js")
def editor_js():
    """ Includes uuidv4() function courtesy of
    https://stackoverflow.com/a/2117523/823020
    """
    js_files = [
        # We require this file here to make sure it is loaded before footnotes.js.
        "wagtailadmin/js/draftail.js",
        "footnotes/js/footnotes.js",
    ]
    js_includes = format_html_join(
        "\n",
        '<script src="{0}"></script>',
        ((static(filename),) for filename in js_files),
    )
    return js_includes


@hooks.register("register_rich_text_features")
def register_footnotes_feature(features):
    """
    Registering the `footnotes` feature, which uses the `FOOTNOTES` Draft.js
    entity type, and is stored as HTML with a
    `<footnotes id="">short-id</footnotes>` tag.
    """
    feature_name = "footnotes"
    type_ = "FOOTNOTES"

    control = {"type": type_, "label": "Fn", "description": "Footnotes"}

    features.register_editor_plugin(
        "draftail",
        feature_name,
        draftail_features.EntityFeature(
            control,
            # The JS for footnotes would have been passed here, see above.
        ),
    )

    features.register_converter_rule(
        "contentstate",
        feature_name,
        {
            "from_database_format": {
                "footnote[id]": FootnotesEntityElementHandler(type_)
            },
            "to_database_format": {
                "entity_decorators": {type_: footnotes_entity_decorator}
            },
        },
    )


def footnotes_entity_decorator(props):
    """
    Draft.js ContentState to database HTML.
    Converts the FOOTNOTES entities into a footnote tag.
    """
    return DOM.create_element("footnote", {"id": props["footnote"]}, props["children"])


class FootnotesEntityElementHandler(InlineEntityElementHandler):
    """
    Database HTML to Draft.js ContentState.
    Converts the footnote tag into a FOOTNOTES entity, with the right data.
    """

    mutability = "IMMUTABLE"

    def get_attribute_data(self, attrs):
        """
        Take the ``footnote UUID`` value from the ``id`` HTML attribute.
        """
        return {"footnote": attrs["id"]}



# Generated by Django 2.0.10 on 2019-02-13 11:23

import django.db.models.deletion
import modelcluster.fields
import wagtail.core.fields
import wagtail_footnotes.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("wagtailcore", "0041_group_collection_permissions_verbose_name_plural")
    ]

    operations = [
        migrations.CreateModel(
            name="Footnote",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "uuid",
                    wagtail_footnotes.fields.CustomUUIDField(
                        help_text="The ID of the footnote is shown in the rich text editor for reference.",
                        unique=True,
                        verbose_name="ID",
                    ),
                ),
                ("text", wagtail.core.fields.RichTextField()),
                (
                    "page",
                    modelcluster.fields.ParentalKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="footnotes",
                        to="wagtailcore.Page",
                    ),
                ),
            ],
        )
    ]



from django.db import models
from modelcluster.fields import ParentalKey
from wagtail.admin.edit_handlers import FieldPanel, InlinePanel
from wagtail.core.fields import RichTextField

from .fields import CustomUUIDField
from .widgets import ReadonlyUUIDInput


class Footnote(models.Model):
    """
    Footnote has a UUID field which is set using JavaScript on object creation
    so that it is available immediately for hardcoding a reference to the
    footnote in the rich text of the page.
    """

    page = ParentalKey("wagtailcore.Page", related_name="footnotes")
    uuid = CustomUUIDField(
        unique=True,
        verbose_name="ID",
        help_text="The ID of the footnote is shown in the rich text editor for "
        "reference.",
    )
    text = RichTextField(features=["bold", "italic", "link"])

    panels = [FieldPanel("text"), FieldPanel("uuid", widget=ReadonlyUUIDInput)]



from django.db.models import UUIDField


class CustomUUIDField(UUIDField):
    """ Returns database value as a string, so that dashes are preserved. """

    def from_db_value(self, value, expression, connection, context=None):
        return str(value)



from django.forms import HiddenInput
from wagtail.utils.widgets import WidgetWithScript


class ReadonlyUUIDInput(WidgetWithScript, HiddenInput):
    """
    This isn't really read-only. It's a hidden input with an an adjacent div
    showing the current value; that way we can set the value in JavaScript, but
    the user can't easily change it.
    """

    def render_html(self, name, value, attrs):
        """Render the HTML (non-JS) portion of the field markup"""
        hidden = super(WidgetWithScript, self).render(name, value, attrs)
        display_value = value[:6] if value is not None else value
        shown = f'<div id="{attrs["id"]}_display-value" style="padding-top: 1.2em;">{display_value}</div>'
        return shown + hidden

    def render_js_init(self, id_, name, value):
        return f'setUUID("{id_}");'
